import control.Controller;
import javafx.application.Application;
import javafx.scene.Scene;
import javafx.stage.Stage;
import model.Model;
import view.View;

public class Baballe extends Application {

    private static int width;
    private static int height;

    public static void main(String[] args) {
        if (args.length == 0) {
            System.err.println("program arguments: width [height]");
            System.err.println("if height not provided, considered to be equal to width");
        }
        try {
            width = Integer.parseInt(args[0]);
            if (args.length == 2) {
                height = Integer.parseInt(args[1]);
            }
            else {
                height = width;
            }
        }
        catch(NumberFormatException e) {
            System.err.println("program arguments: width [height]");
            System.err.println("if height not provided, considered to be equal to width");
        }
        launch(args);
    }

    // NB: stage parameter represents the app window
    @Override
    public void start(Stage stage) throws Exception {
        // create model
        Model model = new Model(width, height);
        // create view, providing model
        View view = new View(model);
        // create control, providing model & view
        Controller control = new Controller(model,view);

        stage.setTitle("Baballe");
        // create the javafx scene for the stage (i.e. the app window)
        // the scene must take as a first parameter a javafx node that is able to contain other types of node
        // in this case, it is a Pane object. (cf. View.java)
        Scene scene = new Scene(view.getRootPane(), view.getWidth(), view.getHeight());

        // attach the scene to the app window
        stage.setScene(scene);
        // show the app window
        stage.show();

        // be sure that keyboard is focused on the view Pane so that
        // keyboard events are considered to come from it.
        view.getRootPane().setFocusTraversable(true);
        view.getRootPane().requestFocus();

        // start animation (i.e. refreshing the scene regularly)
        control.startAnimation();

    }
}
