package control;

import model.Model;
import view.View;

import java.util.Calendar;
import java.util.Random;

public class Controller {

    private static Random loto = new Random(Calendar.getInstance().getTimeInMillis());
    protected Model model;
    protected View view;
    protected ControllerKeyboard keyboard;
    protected ControllerAnimation animator;

    public Controller(Model model, View view) {
        this.model = model;
        this.view = view;
        keyboard = new ControllerKeyboard(model,view,this);
        animator = new ControllerAnimation(model, view, this);

    }

    public void startAnimation() {
        animator.start();
    }
    public void stopAnimation() {
        animator.stop();
    }

    public void step() {
        checkCollisions();
        model.update();
        view.update();
        model.resetFlags();
    }

    public void pushBallLeft() {
        model.getBall().push(180);
    }
    public void pushBallRight() {
        model.getBall().push(0);
    }
    public void pushBallUp() {
        model.getBall().push(90);
    }
    public void pushBallDown() {
        model.getBall().push(270);
    }
    public void strikeBall() {
        model.getBall().push(loto.nextInt(360));
    }

    public void stopBall() {
        model.getBall().stop();
    }

    public void increaseSpeed() {
        model.getBall().increaseSpeed(4.0);
    }
    public void decreaseSpeed() {
        model.getBall().decreaseSpeed(4.0);
    }


    /*
    Rq : la détection de collision peut se faire de façon "préventive" ou bien avec
    l'état courant. En préventif, on vérifie la prochaine position de l'élément, ce qui permet d'éviter
       qu'il vienne chevaucher un autre élément, donnant l'impression qu'il rentre dedans.
       Cette approche pose problème avec les éléments se déplaçant vite : ils semblent rebondir avant
       d'avoir rencontré le futur obstacle. Pour avoir un rendu parfait, il faut déterminer les coordonnées précises
       du point de collision, ce qui peut s'avérer très compliqué et couteux en temps d'exécution quand il y a
       plusieurs objets mouvants impliqués.
       La méthode ci-dessous ne s'embête pas avec ces considérations et se contente de changer la direction de déplacement.

     */
    public void checkCollisions() {
        double[] nextPosBall = model.getBall().getNextPosition();
        view.getBallLook().setHitBoxPosition(nextPosBall[0], nextPosBall[1]);
        // check collisions between ball1 & walls:
        if ( (view.getBallLook().checkCollision(view.getNorthWall())) ||
            (view.getBallLook().checkCollision(view.getSouthWall())) ) {
            model.getBall().setCollisionOccurred(true);
            model.getBall().reverseYSpeed();
        }
        else if ( (view.getBallLook().checkCollision(view.getWestWall())) ||
                (view.getBallLook().checkCollision(view.getEastWall())) ) {
            model.getBall().setCollisionOccurred(true);
            model.getBall().reverseXSpeed();
        }
    }
}
