package control;

import javafx.animation.AnimationTimer;
import model.Model;
import view.View;

public class ControllerAnimation {

    protected Model model;
    protected View view;
    protected Controller control;
    protected AnimationTimer animator;

    public ControllerAnimation(Model model, View view, Controller control) {
        this.model = model;
        this.view = view;
        this.control = control;

        // create an animation timer that calls controller step() method regularly
        // It consists in defining a method, in the present case, with the lambda expression syntax
        // This method will be called very often by the javafx engine, but not mandatory at a fix rhythm
        // This is why this method takes a time (long value) as a parameter which represents the time elapsed in µs
        // since the timer is started. Thanks to this value, we are able to compute the gap of time between the current method
        // call and the previous ones. If the gap becomes larger than a desired value (retrieved with in model.getFrameGap())
        // it is time to a new 'controller frame' by calling the step() method.
        animator = new AnimationTimer(){

            @Override
            public void handle(long time) {

                if (model.getLastFrame() == -1) {
                    model.setLastFrame(time);
                }
                // if not enough time has passed from the previous call, do nothing
                else if ((time-model.getLastFrame()) < model.getFrameGap()) {
                    return;
                }
                // if enough time passed, update the frame time then do a controller step
                model.setLastFrame(time);
                control.step();
            }
        };
    }

    public void start() {
        animator.start();
    }
    public void stop() {
        animator.stop();
    }
}
