package model;

public class BallModel {

    private final double baseSpeed = 4;
    private double speed; // the absolute speed in 1D
    private double speedX;
    private double speedY;
    private double angle; // for convenience, not to recompute it from speedX and speedY;
    // position in absolute space (NB: scene space = absolute space for convenience)
    private double x;
    private double y;
    // flags for different events
    private boolean positionChanged;
    private boolean collisionOccurred; // true when a collision just occurred
    private boolean endCollisionOccurred; // true when counterCollision comes back to 0
    // counter to keep the state collisionOccurred during 10 frames
    private int counterCollision;

    public BallModel() {
        speed = 0.0;
        speedX = 0.0;
        speedY = 0.0;
        x = 0;
        y = 0;
        angle = 0.0;
        positionChanged = false;
        collisionOccurred = false;
        counterCollision = 0;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public double getSpeed() {
        return speed;
    }

    public boolean isPositionChanged() {
        return positionChanged;
    }

    public boolean isCollisionOccurred() {
        return collisionOccurred;
    }

    public void setCollisionOccurred(boolean collisionOccurred) {
        this.collisionOccurred = collisionOccurred;
        counterCollision = 10;
    }

    public boolean isEndCollisionOccurred() {
        return endCollisionOccurred;
    }

    public void resetFlags() {
        positionChanged = false;
        collisionOccurred = false;
        endCollisionOccurred = false;
    }

    public void put(double x, double y) {
        if ((this.x != x) || (this.y != y)) {
            positionChanged = true;
            this.x = x;
            this.y = y;
        }
    }

    public void move() {
        if ((speedX != 0.0) || (speedY != 0.0)) {
            positionChanged = true;
            x += speedX;
            y += speedY;
        }
    }

    public double[] getNextPosition() {
        double[] tab = {x+speedX, y+speedY};
        return tab;
    }

    public void stop() {
        speedX = 0.0;
        speedY = 0.0;
    }

    public void push(double angle) {
        if (speed == 0.0) speed = baseSpeed;
        this.angle = angle;
        updateXYSpeed();
    }

    private void updateXYSpeed() {
        speedX = Math.cos(2.0*Math.PI*angle/360.0)*speed;
        speedY = -Math.sin(2.0*Math.PI*angle/360.0)*speed;
    }

    public void increaseSpeed(double inc) {
        speed += inc;
        if (speed > 50) {
            speed = 50;
        }
        updateXYSpeed();
    }
    public void decreaseSpeed(double inc) {
        speed -= inc;
        if (speed < 0) {
            speed = 0;
        }
        updateXYSpeed();
    }

    public void update() {
        move();
        counterCollision--;
        if ( counterCollision == 0) {
            endCollisionOccurred = true;
        }
    }

    public void reverseXSpeed() {
        if (angle <= 180) {
            push(180-angle);
        }
        else {
            push(540-angle);
        }
    }
    public void reverseYSpeed() {
        push(360-angle);
    }
}
