package view;

import javafx.geometry.Bounds;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Shape;
import model.BallModel;

public class BallLook {
    // Group that contains all shpaes necessary to represent the ball
    private Group body;
    // shapes that constitute the body of the ball
    private Circle outer;
    private Circle inner;

    // Group that contains all shapes necessary to represent the bounding box of the ball
    private Group hitBox;
    // shapes that constitute the hitBox
    private Circle hitCircle;

    // the model associated to this look
    private BallModel ballModel;

    public BallLook(BallModel ballModel, double radius) {

        this.ballModel = ballModel;
        // define the body
        body = new Group();
        outer = new Circle();
        outer.setRadius(radius);
        outer.setFill(Color.BLACK);
        outer.setCenterX(radius);
        outer.setCenterY(radius);
        inner = new Circle();
        inner.setRadius(5);
        inner.setFill(Color.WHITE);
        inner.setCenterX(radius);
        inner.setCenterY(radius);
        body.getChildren().addAll(outer, inner);
        // define the hitbox : in the present case, a circle that matches outer boundaries.
        hitBox = new Group();
        hitCircle = new Circle();
        hitCircle.setRadius(radius);
        hitCircle.setFill(Color.TRANSPARENT);
        hitCircle.setCenterX(radius);
        hitCircle.setCenterY(radius);
        hitBox.getChildren().addAll(hitCircle);
    }

    public Group getBody() {
        return body;
    }

    public void setHitBoxPosition(double x, double y) {
        hitBox.setTranslateX(x);
        hitBox.setTranslateY(y);
    }
    public void update() {
        if (ballModel.isPositionChanged()) {
            body.setTranslateX(ballModel.getX());
            body.setTranslateY(ballModel.getY());
            hitBox.setTranslateX(ballModel.getX());
            hitBox.setTranslateY(ballModel.getY());
        }
        if (ballModel.isCollisionOccurred()) {
            outer.setFill(Color.RED);
        }
        if (ballModel.isEndCollisionOccurred()) {
            outer.setFill(Color.BLACK);
        }
    }

    // check collision between this ball and another shape
    // It is done only using the shapes that constitute the hitbox, instead of checking all
    public boolean checkCollision(Shape s) {
        // get every node in the hitbox group
        for(Node n : hitBox.getChildren()) {
            // must cast retreive node to a shape (NB: always works since there are only shapes in hitbox)
            Shape box = (Shape)n;
            // use javafx method to compute the intersection between two shapes
            Shape inter = Shape.intersect(box, s);
            // get bounds of the intersection
            Bounds b = inter.getBoundsInParent();
            // if bounds width exists => intersection => collision
            if (b.getWidth() != -1) return true;
        }

        return false;
    }
}
